/*
 * Copyright (c) 1990 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * masync - An example of SIGIO user interaction.  When middle C is hit,
 * a scale is played.
 */
  
#include <stdio.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <sundev/midi.h>

int midifd;
int playit;

sigio_handler()
{
	int cc;
	struct midi_msg m;

	printf("caught SIGIO\n");
	while (1) {
		cc = read(midifd, (char *)&m, sizeof m);
		if (cc < 0) {
			perror("read");
			exit(1);
		}
		if (cc == 0)
			return;
		if ((m.mm_data[0] & 0xf0) == 0x90 && m.mm_data[2] != 0 &&
		    m.mm_data[1] == 0x3c) {
			printf("playing scale!\n");
			playit = 1;
		}
	}
}

struct midi_msg scale[] = {
	250, { 0x90, 0x40, 0x40 },
	500, { 0x90, 0x40, 0x0 },
	500, { 0x90, 0x42, 0x40 },
	750, { 0x90, 0x42, 0x0 },
	750, { 0x90, 0x44, 0x40 },
	1000, { 0x90, 0x44, 0x0 },
	1000, { 0x90, 0x45, 0x40 },
	1250, { 0x90, 0x45, 0x0 },
	1250, { 0x90, 0x47, 0x40 },
	1500, { 0x90, 0x47, 0x0 },
	1500, { 0x90, 0x49, 0x40 },
	1750, { 0x90, 0x49, 0x0 },
	1750, { 0x90, 0x4b, 0x40 },
	2000, { 0x90, 0x4b, 0x0 },
	2000, { 0x90, 0x4c, 0x40 },
	2250, { 0x90, 0x4c, 0x0 },
};

usage()
{
	fprintf(stderr, "Usage: masyc [-p port]\n");
	exit(1);
}


main(argc,argv)
	char *argv[];
{
	int fd, cc, i, port, c;
	u_long flags;

	extern char *optarg;
	extern int optind;

	port = 1;
	while ((c = getopt(argc, argv, "p:")) != -1) {
		switch(c) {

		case 'p':
			port = atoi(optarg);
			break;

		default:
			usage();
		}
	}
	if (optind != argc)
		usage();

	signal(SIGIO, sigio_handler);
	midifd = fd = midi_open(O_RDWR, port);

	if (fcntl(fd, F_SETFL, FASYNC) == -1) {
		perror("F_SETFL");
		exit(1);
	}
	while (1) {
		u_long zero = 0;

		sigpause(0);
		if (!playit)
			continue;
		playit = 0;

		if (ioctl(fd, BMDSTIME, (caddr_t)&zero) < 0) {
			perror("BMDSTIME");
			exit(1);
		}
		cc = write(fd, (caddr_t)&scale, sizeof scale);
		if (cc < 0) {
			perror("write");
			exit(1);
		}
	}
}
