/*
 * Copyright (c) 1990 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <sundev/midi.h>

#ifndef lint
static char rcsid[] =
    "@(#) $Header: mr.c,v 1.2 91/08/20 22:14:36 mccanne Exp $";
#endif

u_char midi_msglen[] = {
	0, 0, 0, 0, 0, 0, 0, 0,	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,	0, 0, 0, 0, 0, 0, 0, 0,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, /* 0x80 */
	3, 3, 3, 3, 3, 3, 3, 3,	3, 3, 3, 3, 3, 3, 3, 3, /* 0x90 */
	3, 3, 3, 3, 3, 3, 3, 3,	3, 3, 3, 3, 3, 3, 3, 3, /* 0xa0 */
	3, 3, 3, 3, 3, 3, 3, 3,	3, 3, 3, 3, 3, 3, 3, 3, /* 0xb0 */
	2, 2, 2, 2, 2, 2, 2, 2,	2, 2, 2, 2, 2, 2, 2, 2, /* 0xc0 */
	2, 2, 2, 2, 2, 2, 2, 2,	2, 2, 2, 2, 2, 2, 2, 2, /* 0xd0 */
	3, 3, 3, 3, 3, 3, 3, 3,	3, 3, 3, 3, 3, 3, 3, 3, /* 0xe0 */
	0xff, 1, 3, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0xf0 */
};

int cflag;

pbyte(b)
	int b;
{
	int i;

	static int n = 0;
	static char linebuf[8];

	linebuf[n++] = b;
	printf("%02x", b);
	if (n == 8) {
		if (cflag) {
			putchar('\t');
			for (i = 0; i < 8; ++i)
				if (isprint(linebuf[i]))
					putchar(linebuf[i]);
				else
					putchar('.');
		}
		putchar('\n');
		n = 0;
	} else
		putchar(' ');
}

usage()
{
	fprintf(stderr, "usage: mr [-p port] [-acr]\n");
	exit(1);
}

int
msglen(cp)
	u_char *cp;
{
	int i;

	if (*cp != 0xf0) {
		i = midi_msglen[*cp];
		if (i <= 3)
			return i;
	}
	for (i = 1; i < BMD_MAXMSGLEN; ++i) {
		if (cp[i] == 0xf7)
			return i + 1;
	}
	return i;
}

pbuf(cp, len)
	register u_char *cp;
	register int len;
{
	while (--len >= 0) 
		pbyte(*cp++);
	fflush(stdout);
}

pmsg(p, len)
	struct midi_msg *p;
	register int len;
{
	register u_char *cp;
	register char *s;

	if ((p->mm_data[0] & 0x80) == 0) {
		printf("%08d %02x?\n", p->mm_time, p->mm_data[0]);
		return;
	}
	--len;
	printf("%08d %02x ", p->mm_time, p->mm_data[0]);
	if (len > 10) {
		s = "...";
		len = 10;
	} else
		s = 0;
	cp = &p->mm_data[1];
	while (--len >= 0)
		printf("%02x ", *cp++);
	if (s != 0)
		printf("...");
	putchar('\n');
}

main(argc, argv)
	int argc;
	char **argv;
{
	int fd, cc, c, port;
	int aflag = 0;
	int rflag = 0;
	struct midibuf mb;
	struct midi_msg *p;

	extern char *optarg;

	port = 1;
	while ((c = getopt(argc, argv, "acp:r")) != -1) {
		switch(c) {

		case 'a':
			++aflag;
			break;

		case 'c':
			++cflag;
			break;

		case 'r':
			++rflag;
			break;

		case 'p':
			port = atoi(optarg);
			break;

		default:
			usage();
		}
	}
	fd = midi_open(O_RDONLY, port);
	if (aflag && ioctl(fd, BMDFILTER, 0) < 0) {
		perror("BMDFILTER");
		exit(1);
	}
	while (1) {
		cc = read(fd, (char *)&mb, sizeof mb);
		if (cc < 0) {
			perror("read");
			exit(-1);
		}
		if (cc == 0)
			continue;
		p = (struct midi_msg *)&mb;
		while (cc >= sizeof(*p)) {
			int len = msglen(p->mm_data);
			if (len > cc)
				abort();
			if (rflag)
				pbuf(p->mm_data, len);
			else
				pmsg(p, len);
			len = BMDALIGN(len + 4);
			cc -= len;
			p += len / sizeof(*p);
		}
	}
}
