/*
 * Copyright (c) 1990 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <sys/types.h>
#include <sundev/midi.h>

#include "readline.h"
#include "setting.h"
#include "event.h"
#include "etime.h"
#include "click.h"
#include "track.h"

struct event *
build_playlist(start, end)
	u_long start;
	u_long end;
{
	struct event *play_list, *pending, *p, **pp;

	for (p = master_list; p != 0; p = p->next) 
		if (p->m.mm_time >= start)
			break;

	pp = &play_list;
	pending = 0;
	while (p && p->m.mm_time <= end)
	{
		u_long t = warp_time(s.tempo, p->m.mm_time);
		if (pending && pending->m.mm_time <= t)
		{
			*pp = pending;
			pending = pending->pqnext;
		}
		else if (p->trk->mute > 0)
		{
			p = p->next;
			continue;
		}
		else
		{
			*pp = ealloc();
			*(long *)(*pp)->m.mm_data = *(long *)p->m.mm_data;
			(*pp)->m.mm_time = t;
			(*pp)->m.mm_cmd = ((*pp)->m.mm_cmd & 0xf0) |
				p->trk->channel;
			if (evt_noteon_p(p))
			{
				struct event *tmp = ealloc();
				*(long *)tmp->m.mm_data =
					*(long *)(*pp)->m.mm_data;
				tmp->m.mm_data[2] = 0;
				tmp->m.mm_time = warp_time(s.tempo,
							   p->m.mm_time +
							   p->duration);
				pq_insert(&pending, tmp);
			}
			p = p->next;
		}
		pp = &(*pp)->next;
	}
	for (; pending; pending = pending->pqnext)
	{
		*pp = pending;
		pp = &(*pp)->next;
	}
	*pp = 0;

	return play_list;

}

int
run_command(start, trk)
	int start;
	struct track *trk;
{
	struct event *click;
	struct track *curr;	
	extern struct event *midi_run(); /* XXX */
	
	if (trk) {
		save_mutes();
		mute_all();
		trk->mute = 0;
	}
	/*
	 * We could get a Ctrl-C (between here and midi_run)
	 * that screws up the mutes.  That's not too likely.
	 */
	efreel(raw_list);
	raw_list = midi_run(warp_time(s.tempo, start), 
			    warp_time(s.tempo, ~0),
			    build_playlist(start, ~0),
			    first_click(start));
	if (trk) 
		restore_mutes();
}

